<?php
/**
 * Universal Products Importer
 */

defined ( 'ABSPATH' ) || exit;

if ( ! class_exists ( 'WP_Importer' ) ) {
	return;
}

class WPCE_Universal_Products_Importer extends WP_Importer {

	/**
	 * The current file id.
	 */
	public $id;

	/**
	 * The current file url.
	 */
	public $file_url;

	/**
	 * The current import page.
	 */
	public $import_page;
	
	/**
	 * The id of current finder.
	 */
	public $finder_id = '';

	/**
	 * The current delimiter.
	 */
	public $delimiter;
	
	/**
	 * Whether to skip existing terms
	 */
	public $delete_existing = false;

	/**
	 * Constructor
	 */
	public function __construct () {
		$this->import_page		= 'wpce_universal_products_importer';
		$this->finder_id		= isset ( $_REQUEST['finder_id'] ) ? absint ( $_REQUEST['finder_id'] ) : '';
		$this->delimiter		= empty ( $_POST['delimiter'] ) ? ',' : ( string ) wc_clean ( wp_unslash ( $_POST['delimiter'] ) );
		$this->delete_existing	= empty ( $_POST['delete_existing'] ) ? '0' : ( string ) wc_clean ( wp_unslash ( $_POST['delete_existing'] ) );
	}

	/**
	 * Registered callback function for the WordPress Importer.
	 *
	 * Manages the three separate stages of the CSV import process.
	 */
	public function dispatch () {
		$this->header ();

		if ( $this->finder_id ) {
			$step = empty ( $_GET['step'] ) ? 0 : ( int ) $_GET['step'];

			switch ( $step ) {
				case 0:
					$this->greet ();
				break;

				case 1:
					check_admin_referer ( 'import-upload' );

					if ( $this->handle_upload() ) {
						if ( $this->id ) {
							$file = get_attached_file ( $this->id );
						} else {
							$file = ABSPATH . $this->file_url;
						}

						add_filter ( 'http_request_timeout', array ( $this, 'bump_request_timeout' ) );

						$this->import( $file );
					}
				break;
			}
		} else {
			echo '
				<p>
					<strong>' . __( 'Sorry, there has been an error.', 'wpce' ) . '</strong><br/>
					' . __( 'You need to select a Finder first through <strong>WPCE</strong> menu.', 'wpce' ) . '
				</p>
			';
		}
		
		$this->footer ();
	}

	/**
	 * Import is starting.
	 */
	private function import_start () {
		if ( function_exists ( 'gc_enable' ) ) {
			gc_enable ();
		}
		
		wc_set_time_limit ( 0 );
		@ob_flush ();
		@flush ();
		@ini_set ( 'auto_detect_line_endings', '1' );
	}

	/**
	 * UTF-8 encode the data if `$enc` value isn't UTF-8.
	 */
	public function format_data_from_csv ( $data, $enc ) {
		return ( 'UTF-8' === $enc ) ? $data : utf8_encode ( $data );
	}

	/**
	 * Import the file if it exists and is valid.
	 */
	public function import ( $file ) {
		if ( ! is_file ( $file ) ) {
			$this->import_error ( __( 'The file does not exist, please try again.', 'wpce' ) );
		}

		$this->import_start ();
		
		$handle	= fopen ( $file, 'r' );
		
		$universal_product_ids			= array ();
		if ( ! $this->delete_existing ) {
			$universal_product_ids		= wpce_get_finder_universal_relationships ( $this->finder_id );
			
			if ( empty ( $universal_product_ids ) ) {
				$universal_product_ids	= array ();
			}
		}
		
		if ( false !== $handle ) {
			$header = fgetcsv ( $handle, 0, $this->delimiter );

			if ( 1 === count ( $header ) ) {
				$product_skus = array ();
				$row = fgetcsv ( $handle, 0, $this->delimiter );

				while ( false !== $row ) {
					list ( $skus ) = $row;
					
					if ( ! empty ( $skus ) ) {
						$skus = explode ( ',', $skus );
						$product_skus = array_merge ( $product_skus, $skus );
					}

					$row = fgetcsv ( $handle, 0, $this->delimiter );
				}
				
				if ( ! empty ( $product_skus ) ) {
					$product_skus = array_map ( 'trim', $product_skus );
					$product_skus = array_unique ( $product_skus );
					
					global $wpdb;
					
					$product_ids = $wpdb->get_col ( "SELECT `post_id` FROM $wpdb->postmeta WHERE `meta_key` = '_sku' AND `meta_value` IN ( " . "'" . implode ( "','", $product_skus ) . "'" . " )" );
					
					$universal_product_ids = array_merge ( $universal_product_ids, $product_ids );
					$universal_product_ids = array_unique ( $universal_product_ids );
				}
			} else {
				$this->import_error ( __( 'The CSV is invalid.', 'wpce' ) );
			}

			fclose ( $handle );
		}
		
		update_post_meta ( $this->finder_id, 'wpce_universal_products', $universal_product_ids );
		
		// Show Result.
		echo '
			<div class="updated settings-error">
				<p>';
					printf (
						esc_html__( 'Import complete - imported %s universal products.', 'wpce' ),
						'<strong>' . ( isset ( $product_skus ) ? count ( $product_skus ) : 0 ) . '</strong>'
					);
		echo '
				</p>
			</div>
		';

		$this->import_end ();
	}

	/**
	 * Performs post-import cleanup of files and the cache.
	 */
	public function import_end () {
		echo '<p>' . esc_html__( 'All done!', 'wpce' ) . '</p>';

		do_action ( 'import_end' );
	}

	/**
	 * Handles the CSV upload and initial parsing of the file to prepare for.
	 * displaying author import options.
	 */
	public function handle_upload () {
		$file_url = isset ( $_POST['file_url'] ) ? wc_clean ( wp_unslash ( $_POST['file_url'] ) ) : '';

		if ( empty ( $file_url ) ) {
			$file = wp_import_handle_upload ();

			if ( isset ( $file['error'] ) ) {
				$this->import_error ( $file['error'] );
			}

			if ( ! wc_is_file_valid_csv ( $file['file'], false ) ) {
				// Remove file if not valid.
				wp_delete_attachment ( $file['id'], true );

				$this->import_error ( __( 'Invalid file type. The importer supports CSV and TXT file formats.', 'wpce' ) );
			}

			$this->id = absint ( $file['id'] );
		} else if ( file_exists ( ABSPATH . $file_url ) ) {
			if ( ! wc_is_file_valid_csv ( ABSPATH . $file_url ) ) {
				$this->import_error ( __( 'Invalid file type. The importer supports CSV and TXT file formats.', 'wpce' ) );
			}

			$this->file_url = esc_attr ( $file_url );
		} else {
			$this->import_error ();
		}

		return true;
	}

	/**
	 * Output header html.
	 */
	public function header () {
		?><div class="wrap">
			<?php if ( $this->finder_id ) { ?>
				<h1 class="wp-heading-inline"><?php printf ( __( '%s - Import Universal Products', 'wpce' ), get_the_title ( $this->finder_id ) ); ?></h1>
				
				<a href="edit.php?post_type=wpce&page=wpce_manage_terms&finder_id=<?php echo $this->finder_id; ?>" class="page-title-action"><?php _e( 'Manage Terms', 'wpce' ); ?></a>
				<hr class="wp-header-end" />
			<?php } else { ?>
				<h1><?php _e( 'WPCE - Import Universal Products', 'wpce' ); ?></h1>
			<?php } ?>
		<?php
	}

	/**
	 * Output footer html.
	 */
	public function footer () {
		?></div><?php
	}

	/**
	 * Output information about the uploading process.
	 */
	public function greet () {

		echo '<div class="narrow">';
			echo '<p>' . esc_html__( 'Hi there! Upload a CSV file containing universal products sku to import into your shop. Choose a .csv file to upload, then click "Upload file and import".', 'wpce' ) . '</p>';
			
			/* translators: 1: Link to tax rates sample file 2: Closing link. */
			echo '<p>' . sprintf ( esc_html__( 'Your CSV needs to include columns in a specific order. %1$sClick here to download a sample%2$s.', 'wpce' ), '<a href="' . WPCE_PLUGIN_URL . '/sample-data/sample_universal_products.csv">', '</a>' ) . '</p>';

			$action = 'admin.php?import=' . $this->import_page . '&finder_id=' . $this->finder_id . '&step=1';

			$bytes		= apply_filters ( 'import_upload_size_limit', wp_max_upload_size () );
			$size		= size_format ( $bytes );
			$upload_dir	= wp_upload_dir ();
			
			if ( ! empty( $upload_dir['error'] ) ) :
				?><div class="error">
					<p><?php esc_html_e( 'Before you can upload your import file, you will need to fix the following error:', 'wpce' ); ?></p>
					<p><strong><?php echo esc_html ( $upload_dir['error'] ); ?></strong></p>
				</div><?php
			else :
				?><form enctype="multipart/form-data" id="import-upload-form" method="post" action="<?php echo esc_attr ( wp_nonce_url ( $action, 'import-upload' ) ); ?>">
					<table class="form-table">
						<tbody>
							<tr>
								<th>
									<label for="upload"><?php esc_html_e( 'Choose a file from your computer:', 'wpce' ); ?></label>
								</th>
								
								<td>
									<input type="file" id="upload" name="import" size="25" />
									<input type="hidden" name="action" value="save" />
									<input type="hidden" name="max_file_size" value="<?php echo absint ( $bytes ); ?>" />
									
									<small>
										<?php
											printf (
												/* translators: %s: maximum upload size */
												esc_html__( 'Maximum size: %s', 'wpce' ),
												esc_attr ( $size )
											);
										?>
									</small>
								</td>
							</tr>
							
							<tr>
								<th>
									<label for="file_url"><?php esc_html_e( 'OR enter path to file:', 'wpce' ); ?></label>
								</th>
								
								<td>
									<?php echo ' ' . esc_html ( ABSPATH ) . ' '; ?><input type="text" id="file_url" name="file_url" size="25" />
								</td>
							</tr>
							
							<tr>
								<th><label><?php esc_html_e( 'Delimiter', 'wpce' ); ?></label><br/></th>
								<td><input type="text" name="delimiter" placeholder="," size="2" /></td>
							</tr>
							
							<tr>
								<th><label><?php esc_html_e( 'Delete existing universal products', 'wpce' ); ?></label><br/></th>
								<td>
									<input type="checkbox" name="delete_existing" value="1" />
								</td>
							</tr>
						</tbody>
					</table>
					
					<p class="submit">
						<button type="submit" class="button" value="<?php esc_attr_e( 'Upload file and import', 'wpce' ); ?>"><?php esc_html_e( 'Upload file and import', 'wpce' ); ?></button>
					</p>
				</form><?php
			endif;

		echo '</div>';
	}

	/**
	 * Show import error and quit.
	 */
	private function import_error ( $message = '' ) {
		echo '<p>';
			echo '<strong>' . esc_html__( 'Sorry, there has been an error.', 'wpce' ) . '</strong><br />';
			
			if ( $message ) {
				echo esc_html ( $message );
			}
		echo '</p>';
		
		$this->footer ();
		die ();
	}

	/**
	 * Added to http_request_timeout filter to force timeout at 60 seconds during import.
	 */
	public function bump_request_timeout ( $val ) {
		return 60;
	}
}
